import 'dart:convert';

import 'src/native_bridge.dart';
import 'src/private_user.dart';
import 'usermanagement/ban_info.dart';
import 'usermanagement/conflict_user.dart';
import 'usermanagement/identity.dart';
import 'usermanagement/user_update.dart';

class CurrentUser extends PrivateUser {
  CurrentUser.fromJSON(Map<String, dynamic> json) : super.fromJSON(json);

  ///
  /// Requests a bulk change of properties for the current user.
  ///
  /// [update] New user details.
  ///
  Future updateDetails(UserUpdate update) {
    return NativeBridge.async(
        'CurrentUser.updateDetails', jsonEncode(update.toJSON()));
  }

  ///
  /// Adds Identity for the specified provider.
  ///
  /// [identity] Identity to be added.
  /// [onSuccess]  A callback to indicate if this operation was successful.
  /// [onConflict] Called if there was a conflict.
  /// [onError]  Called if operation failed.
  ///
  void addIdentity(Identity identity, Function onSuccess,
      Function(ConflictUser) onConflict, Function onError) {
    NativeBridge.async('CurrentUser.addIdentity', jsonEncode(identity.toJSON()))
        .then((value) {
      if (jsonDecode(value)['result'] == '') {
        onSuccess();
      } else {
        onConflict(ConflictUser.fromJSON(jsonDecode(value)));
      }
    }).catchError(onError);
  }

  ///
  /// Removes Identity for the specified provider.
  ///
  /// [providerId] The provider connected to an auth identity on the current user to remove.
  /// Valid provider ids are found in [IdentityProviderIds] in addition to custom providers.
  ///
  Future removeIdentity(String providerId) {
    return NativeBridge.async('CurrentUser.removeIdentity', providerId);
  }

  ///
  /// Returns reason and expiration of the ban of the current user
  ///
  Future<BanInfo?> getBanInfo() async {
    return await NativeBridge.sync('CurrentUser.getBanInfo', '').then((value) {
      if (jsonDecode(value) == null) {
        return null;
      }
      Map<String, dynamic> decoded = jsonDecode(value);
      if (decoded['result'] != null && decoded['result'].isEmpty) {
        return null;
      }
      return BanInfo.fromJSON(decoded);
    });
  }

  ///
  /// Returns if the current user is banned
  ///
  Future<bool> isBanned() {
    return NativeBridge.sync('CurrentUser.isBanned', '')
        .then((value) => jsonDecode(value)['result']);
  }

  ///
  /// Refresh current user properties.
  ///
  Future refresh() {
    return NativeBridge.async('CurrentUser.refresh', null);
  }

  @override
  String toString() {
    return 'CurrentUser{${super.toString()}';
  }
}
